#!/usr/bin/perl

# librarians.cgi - create and maintain MyLibrary librarians

# Eric Lease Morgan <emorgan@nd.edu>


# require the necessary modules
use CGI qw/:standard/;
use CGI::Carp qw(fatalsToBrowser);
use strict;
use MyLibrary::Core;
require 'etc/subroutines.pl';

# initialize two global variables
my $cgi = CGI->new;
my $html = '';

# branch according to the input
if (! $cgi->param('cmd')) { &home }
elsif ($cgi->param('cmd') eq 'create')  { &create }
elsif ($cgi->param('cmd') eq 'find')    { &find }
elsif ($cgi->param('cmd') eq 'edit')    { &edit }
elsif ($cgi->param('cmd') eq 'delete')  { &delete }
else { $html = $cgi->p("Unknown command. Call Eric.") }

# quit
&gracefulExit;


sub header {
	
	# get the value of this script
	my $script = $cgi->script_name;

	# return a standard html page; notice the stylesheet
	return <<EOH;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
	<head>
		<title>Librarians</title>
		<link rel='stylesheet' href='/etc/style-admin.css' type='text/css' />
	</head>
	<body>
		<div id="navcontainer">
			<ul id="navlist">
				<li><a href="home.cgi">Home</a></li>
				<li><a href="facets.cgi">Facets</a></li>
				<li><a href="terms.cgi">Terms</a></li>
				<li><a href="location-types.cgi">Location types</a></li>
				<li><a href="librarians.cgi" id="current">Librarians</a></li>
				<li><a href="resources.cgi">Resources</a></li>
			</ul>
		</div>
	
	<div class='menu'>
		<p><b><a href="$script">Manage librarians</a></b></p>
		<a href="$script?cmd=create">Create</a><br />
		<a href="$script?cmd=find">Find all</a><br />
		<a href="$script?cmd=edit">Edit</a><br />
		<a href="$script?cmd=delete">Delete</a><br />
		<br />
	</div>
	<div class='content'>
EOH

}


sub home {

	# build a simple home page
	$html  = $cgi->h1('Manage librarians');
	$html .= $cgi->p('Use these pages to manage MyLibrary librarians.');

}


sub create {

	# initialize the output
	$html = $cgi->h1('Create');
	
	# branch according to the input
	if (! $cgi->param('button')) {
		
		# build a list of terms
		my @term_names;
		foreach (MyLibrary::Term->get_terms (sort => 'name')) { push @term_names, $_->term_name }

		# build the form
		$html .= $cgi->p('Use this form to create a librarian.');
		$html .= $cgi->start_form(-method => 'post', -action => $cgi->script_name);
		$html .= $cgi->hidden(-name => 'cmd', -value => 'create');
		$html .= table({-border => 0}, Tr({-align => 'left', -valign => 'top'},
						[
							th(['Item',      'Value']),
							td(['Name',      $cgi->textfield(-name => 'name')]),
							td(['Email',     $cgi->textfield(-name => 'email')]),
							td(['Telephone', $cgi->textfield(-name => 'telephone')]),
							td(['URL',       $cgi->textfield(-name => 'url')]),
							td(['Term(s)',   $cgi->scrolling_list(-name =>'term_names', -values => \@term_names, -size => 5, -multiple => 'true')]),
							td([''    ,      $cgi->submit(-name => 'button', -value => 'Create')])
						]
					)
				 );
		$html .= $cgi->endform;
    
    }
    
    else {
    
    	# create an object, fill it, and save
    	my $librarian = MyLibrary::Librarian->new;
    	$librarian->name($cgi->param('name'));
    	$librarian->email($cgi->param('email'));
    	$librarian->telephone($cgi->param('telephone'));
    	$librarian->url($cgi->param('url'));
    	my @term_ids = ();
    	foreach ($cgi->param('term_names')) {
    	
    		my $term = MyLibrary::Term->new(name => $_);
    		push @term_ids, $term->term_id;
    		
    	}
    	$librarian->term_ids(new => [@term_ids]);
		$librarian->commit;
		
		# get the librarian name
		my $librarian = MyLibrary::Librarian->new(id => $librarian->id);
		
		# create the list of terms
		my $term_names;
		foreach ($cgi->param('term_names')) { $term_names .= $_ . '; ' }
		
		# echo the results
		$html .= $cgi->p('Librarian created:');
		$html .= $cgi->ul(li('ID -', $librarian->id),
		                  li('Name - ', $librarian->name),
		                  li('Email - ', $librarian->email),
		                  li('Telephone - ', $librarian->telephone),
		                  li('URL - ', $librarian->url),
		                  li('Term(s) - ', $term_names)
		                 );
    
    }
    
}


sub find {

	# initialize the output
	$html  = $cgi->h1('All librarians');
	$html .= $cgi->p('This is a list of all the librarians:');
	
	# get all the librarians, build a list, and display it
	my @librarians = MyLibrary::Librarian->get_librarians(sort => 'name');
	my $items;
	foreach (@librarians) { $items .= $cgi->li($_->name, ' (' . $_->id . ')') }
	$html .= $cgi->ul($items);
	
}


sub edit {

	# initialize the output and a button
	$html = $cgi->h1('Edit librarians');
	my $submit .= $cgi->submit(-name => 'button', -value => 'Edit');
	
	# branch according to the input
	if (! $cgi->param('button')) {
	
		# create a hash of librarians; needs to be sorted
		my @librarians = MyLibrary::Librarian->get_librarians(sort => 'name');
		my %librarians;
		foreach (@librarians) { $librarians{$_->id} = $_->name }
	
		# display a form
		$html .= $cgi->p('Select a librarian to edit');
		$html .= $cgi->start_form(-method => 'post', -action => $cgi->script_name);
		$html .= $cgi->hidden(-name => 'cmd', -value => 'edit');
		$html .= $cgi->popup_menu(-name => 'id', -values => [sort keys %librarians], -labels => \%librarians);
		$html .= $submit;			 
		$html .= $cgi->endform;

	}
	
	elsif (! $cgi->param('name')) {
	
    	# create an object based on the input, and create form parts
    	my $librarian = MyLibrary::Librarian->new(id => $cgi->param('id'));
    	
		# build a list of terms
		my @term_names;
		foreach (MyLibrary::Term->get_terms (sort => 'name')) { push @term_names, $_->term_name }
		
		# build a list of terms for this librarian
		my @librarian_terms;
		foreach ($librarian->term_ids) {
		
			my $term = MyLibrary::Term->new(id => $_);
			push @librarian_terms, $term->term_name;
		
		}

		# display an edit form
		$html .= $cgi->p('Use this form to edit a term.');
		$html .= $cgi->start_form(-method => 'post', -action => $cgi->script_name);
		$html .= $cgi->hidden(-name => 'cmd', -value => 'edit');
		$html .= $cgi->hidden(-name => 'id', -value => $librarian->id);
		$html .= table({-border => 0},
					Tr({-align => 'left', -valign => 'top'},
						[
							th(['Item',      'Value']),
							td(['Name',      $cgi->textfield(-name => 'name', -value => $librarian->name)]),
							td(['Email',     $cgi->textfield(-name => 'email', -value => $librarian->email)]),
							td(['Telephone', $cgi->textfield(-name => 'telephone', -value => $librarian->telephone)]),
							td(['URL',       $cgi->textfield(-name => 'url', -value => $librarian->url)]),
							td(['Term(s)',   $cgi->scrolling_list(-name =>'term_names', -values => [@term_names], -size => 5, -multiple => 'true', -default => [@librarian_terms])]),
							td([''    ,      $cgi->submit(-name => 'button', -value => 'Edit')])
						]
					)
				 );
		$html .= $cgi->endform;
    
    }
    
	else {

		# create and object, fill it, and save
		my $librarian = MyLibrary::Librarian->new(id => $cgi->param('id'));
		$librarian->name($cgi->param('name'));
		$librarian->email($cgi->param('email'));
		$librarian->telephone($cgi->param('telephone'));
		$librarian->url($cgi->param('url'));
    	my @term_ids = ();
    	foreach ($cgi->param('term_names')) {
    	
    		my $term = MyLibrary::Term->new(name => $_);
    		push @term_ids, $term->term_id;
    		
    	}
		$librarian->term_ids(del => [$librarian->term_ids]);
    	$librarian->term_ids(new => [@term_ids]);
		$librarian->commit;
		
		# get the librarian name
		my $librarian = MyLibrary::Librarian->new(id => $librarian->id);
		
		# create the list of terms
		my $term_names;
		foreach ($cgi->param('term_names')) { $term_names .= $_ . '; ' }
		
		# echo the results
		$html .= $cgi->p('Librarian updated:');
		$html .= $cgi->ul(li('ID -', $librarian->id),
		                  li('Name - ', $librarian->name),
		                  li('Email - ', $librarian->email),
		                  li('Telephone - ', $librarian->telephone),
		                  li('URL - ', $librarian->url),
		                  li('Term(s) - ', $term_names)
		                 );
    
    }

}


sub delete {

	# initialize the output
	$html = $cgi->h1('Delete librarians');
	
	# branch according to the input
	if (! $cgi->param('button')) {
	
		# build of hash of all terms; needs to be sorted
		my @librarians = MyLibrary::Librarian->get_librarians(sort => 'name');
		my %librarians;
		foreach (@librarians) { $librarians{$_->id} = $_->name }

		# display a pop-up list of terms
		$html .= $cgi->p('Select a librarian to delete:');
		$html .= $cgi->start_form(-method => 'post', -action => $cgi->script_name);
		$html .= $cgi->hidden(-name => 'cmd', -value => 'delete');
		$html .= $cgi->popup_menu(-name => 'id', -values => \%librarians);
		$html .= $cgi->submit(-name => 'button', -value => 'delete', -label => 'Delete');		 
		$html .= $cgi->endform;

	}
	
	elsif ($cgi->param('button') eq 'Delete') {
    
    	# create an object based on input
    	my $librarian = MyLibrary::Librarian->new(id => $cgi->param('id'));
    	
		# display a confirmation
		$html .= $cgi->p('Are you sure you want to delete the librarian named <b>' . $librarian->name . '</b>?');
		$html .= $cgi->start_form(-method => 'post', -action => $cgi->script_name);
		$html .= $cgi->hidden(-name => 'cmd', -value => 'delete');
		$html .= $cgi->hidden(-name => 'id', -value => $librarian->id);
		$html .= $cgi->submit(-name => 'button', -value => 'yes', -label => 'Yes');
		$html .= ' ';
		$html .= $cgi->submit(-name => 'button', -value => 'no', -label => 'No');		 
		$html .= $cgi->endform;
    
    }
    
    elsif ($cgi->param('button') eq 'Yes') {
    
    	# delete the term; do the work and echo the result
    	my $librarian = MyLibrary::Librarian->new(id => $cgi->param('id'));
    	$librarian->delete;
    	$html .= $cgi->p('The librarian ' . $librarian->name . ' has been deleted.');
    
    }
    
    elsif ($cgi->param('button') eq 'No') {
    
    	# abort
    	my $librarian = MyLibrary::Librarian->new(id => $cgi->param('id'));
    	$html .= $cgi->p('The librarian ' . $librarian->name . ' has <b>not</b> been deleted.');

    }
    
}


sub gracefulExit {

	# output the 'magic line', a standard header, the content, and a standard footer, then done
	print $cgi->header;
	print &header;
	print $html;
	print &footer;
	exit;

}




