<?xml version='1.0' ?>
<?xml-stylesheet type="text/css" href="http://www.ockham.org/css/main.css" ?>
<?xml-stylesheet type="text/css" href="etc/style.css" ?>
		
<xsl:stylesheet
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:srw_dc="info:srw/schema/1/dc-v1.1"
	xmlns:srw="http://www.loc.gov/zing/srw/"
	xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
	version='1.0'>

	<xsl:output method='html' omit-xml-declaration='yes' />
	<!-- set the number of record per page -->
	<xsl:param name="recordsPerPage">
		<xsl:choose>
			<xsl:when test="@recordsPerPage">
				<xsl:value-of select="@recordsPerPage" />
			</xsl:when>
			<xsl:otherwise>5</xsl:otherwise>
		</xsl:choose>
	</xsl:param>
	
	<!-- current page number -->
	<xsl:param name="pageNumber">
		<xsl:choose>
			<xsl:when test="@pageNumber">
				<xsl:value-of select="@pageNumber" />
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:param>
	
	<!-- query parameters that send to the sru client-->
	<xsl:param name="clientParameters" select="@clientParameters" />
	<xsl:param name="otherParameters" select="@otherParameters" />

	<!-- a template loop for displaying the pages -->
	 <xsl:template name="loop">
		 <xsl:param name="maxcount"/>
		 <xsl:param name="incriment-factor"/>
		 <xsl:param name="initial-value"/>
		 <xsl:if test="$initial-value &lt; number($maxcount)+1 and $maxcount &gt; 1">
			 <xsl:choose>
				 <xsl:when test="number($initial-value)-1 = $pageNumber"> 
					 <xsl:value-of select='$initial-value' />
				 </xsl:when>
				 <xsl:otherwise>
					 <a>
						 <xsl:attribute name="href"><xsl:value-of select="$clientParameters" />&amp;pageNumber=<xsl:value-of select="number($initial-value)-1" />
						 </xsl:attribute>
						 <xsl:value-of select='$initial-value' />
					 </a>
			 	</xsl:otherwise>
	       		</xsl:choose>
			 <xsl:text> </xsl:text>
			 <xsl:call-template name="loop">
				 <xsl:with-param name="maxcount" select="$maxcount"/>
				 <xsl:with-param name="initial-value" select="$initial-value+$incriment-factor"/>
				 <xsl:with-param name="incriment-factor" select="$incriment-factor"/>
			 </xsl:call-template>
		 </xsl:if>
	 </xsl:template>
	 
	 <!-- a template for display keywords hot -->
	 <xsl:template name="keywordsDisplay">
		 <xsl:param name="stringafter"/>
		 <xsl:param name="allstring"/>
		 <xsl:choose>
			 <xsl:when test="substring-after($stringafter, ';')">
				 <a><xsl:attribute name="href"><xsl:value-of select="$otherParameters" />&amp;query=<xsl:value-of select="substring-before($stringafter, ';')" />
				 </xsl:attribute>
			 	 <xsl:value-of select="substring-before($stringafter, ';')" />
			 	</a>
				<xsl:text> </xsl:text>
			 	<xsl:call-template name="keywordsDisplay">
					<xsl:with-param name="allstring" select="concat($allstring, ' and ', substring-before($stringafter, ';'))" />
					<xsl:with-param name="stringafter" select="substring-after($stringafter, ';')" />
       				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<a><xsl:attribute name="href"><xsl:value-of select="$otherParameters" />&amp;query=<xsl:value-of select="$stringafter" />
				     	</xsl:attribute>
					<xsl:value-of select="$stringafter" />
				</a>
				<xsl:text> -- </xsl:text>
				<a><xsl:attribute name="href"><xsl:value-of select="$otherParameters" />&amp;query=<xsl:value-of select="concat(substring($allstring, 6, string-length($allstring)-4), ' and ', $stringafter)" />
					</xsl:attribute>
					<xsl:text>Similar content from current index</xsl:text>
				</a>	
			</xsl:otherwise>
		</xsl:choose>
	 </xsl:template>
	
	<!-- let's get started -->
	<xsl:template match='/'>
	
	<!-- list of the spells -->
		<div class="otherSearches">
			<xsl:if test="srw:searchRetrieveResponse/srw:extraResponseData/srw:spellset/srw:spell">
				<h2>Alternative spellings</h2>
			</xsl:if>
			<xsl:for-each select='srw:searchRetrieveResponse/srw:extraResponseData/srw:spellset/srw:spell'>
				<p>
					<xsl:if test="srw:count &gt; 0">
						<span style='font-weight: bold'><xsl:value-of select='srw:term' /></span><xsl:text>: </xsl:text>
				<xsl:for-each select='srw:suggestions/srw:suggestion'>
					<a><xsl:attribute name="href"><xsl:value-of select="$otherParameters" />&amp;query=<xsl:value-of select="." /></xsl:attribute><xsl:value-of select='.' /></a><xsl:text>; </xsl:text>
				</xsl:for-each>
					</xsl:if>
				</p>
			</xsl:for-each>
			
		<!-- list of the synonyms -->
			<xsl:if test="srw:searchRetrieveResponse/srw:extraResponseData/srw:synonymset/srw:synonym">
				<h2>Possible synonyms</h2>
			</xsl:if>
			<xsl:for-each select='srw:searchRetrieveResponse/srw:extraResponseData/srw:synonymset/srw:synonym'>
				<p>
					<xsl:if test="srw:count &gt; 0">
						<span style='font-weight: bold'><xsl:value-of select='srw:term' /></span><xsl:text>: </xsl:text>
<xsl:for-each select='srw:suggestions/srw:suggestion'>
						<a><xsl:attribute name="href"><xsl:value-of select="$otherParameters" />&amp;query=<xsl:value-of select="." />
						</xsl:attribute>
						<xsl:value-of select='.' /></a><xsl:text>; </xsl:text>
				</xsl:for-each>
					</xsl:if>
					<xsl:if test="srw:count &lt; 1">
						<span style='font-weight: bold'><xsl:value-of select='srw:term' /></span><xsl:text>: </xsl:text>
						<xsl:text>(None)</xsl:text>
						
					</xsl:if>
				</p>
				
			</xsl:for-each>
	</div>		

		<!-- display the reason d' existance -->
		<p>
			<xsl:text>Your search (</xsl:text>
			<xsl:value-of select='srw:searchRetrieveResponse/srw:echoedSearchRetrieveRequest/srw:query'/>
			<xsl:text>) matched </xsl:text>
			<xsl:value-of select='srw:searchRetrieveResponse/srw:numberOfRecords' />
			<xsl:text> record(s). </xsl:text>
			<xsl:if test="srw:searchRetrieveResponse/srw:numberOfRecords &gt; 0">
			</xsl:if>
		</p>

			<!-- list the records -->
		<ol>
			<xsl:for-each select='//srw_dc:dc'>
				<!--	<xsl:if test="position() &gt;=($recordsPerPage*number($pageNumber)+1) and position() &lt;=number($recordsPerPage*number($pageNumber)+$recordsPerPage)">-->
					<li value="{position()+$recordsPerPage*number($pageNumber)}">
					<a> 
						<xsl:attribute name='href'>
							<xsl:value-of select='dc:identifier' />
						</xsl:attribute>
						<xsl:value-of select='dc:title' />
					</a>
					<ul class='detaillist'>
						
						<!--
						<li>
							<span class='bold'>date</span> - 
							<xsl:value-of select='dc:date' />
						</li>
						-->
						
						<li>
							<span class='bold'>creator</span> - 
							<xsl:value-of select='dc:creator' />
						</li>
						
						<!-- 
						<li>
							<span class='bold'>provider</span> - 
							<xsl:value-of select='dc:provider' />
						</li>
						-->
						
						<li>
							<span class='bold'>location</span> - <a>
								<xsl:attribute name='href'>
									<xsl:value-of select='dc:identifier' />
								</xsl:attribute>
								<xsl:value-of select='dc:identifier' />
							</a>
							<xsl:text> -- </xsl:text>
							<a>
								<xsl:attribute name='href'>http://www.google.com/search?hl=en&amp;lr=&amp;q=related:<xsl:value-of select='dc:identifier' />
								</xsl:attribute>
								<xsl:text>Similar pages from Google</xsl:text>
							</a>
									
						</li>
						<li>
							<span class='bold'>description</span> - 
							<xsl:value-of select='dc:description' />
						</li>
						<li>
							<span class='bold'>keywords</span> -
							<xsl:call-template name="keywordsDisplay">
								<xsl:with-param name="stringafter" select="dc:relation"/>
								<xsl:with-param name="allstring" select="''" />
							</xsl:call-template>
										  
						</li>
											 					</ul>
				</li>
				<!--</xsl:if>-->
			</xsl:for-each>
		</ol>
		
		<!-- show previous page, first 25 page number, and next page -->
		<div style='font-size: large; text-align: center; margin-bottom: 1em'>
		<xsl:if test="$pageNumber &gt; 0">
			<a>
				<xsl:attribute name="href"><xsl:value-of select="$clientParameters" />&amp;pageNumber=<xsl:value-of select="number($pageNumber)-1" />
				</xsl:attribute>
				&lt;&lt; Previous
			</a>
		</xsl:if>
		<xsl:variable name="totalRecords" select='srw:searchRetrieveResponse/srw:numberOfRecords' />
		
		<!-- total pages -->
		<xsl:variable name="displayPage1" select="ceiling($totalRecords div $recordsPerPage)" />
		
		<!-- page number should show -->
		<xsl:variable name="displayPage">
			<xsl:choose>
				<xsl:when test="$displayPage1 &gt; 25">
					<xsl:value-of select="25" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="ceiling($totalRecords div $recordsPerPage)" />
				</xsl:otherwise>
			</xsl:choose>	
		</xsl:variable>
		
		<xsl:variable name="difference">
			<xsl:choose>
				<xsl:when test="number($pageNumber) &gt;24">
					<xsl:value-of select="number($pageNumber)-24" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="0" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:call-template name="loop">
			<xsl:with-param name="maxcount" select="$displayPage+$difference"/>
			<xsl:with-param name="incriment-factor" select="1"/>
			<xsl:with-param name="initial-value" select="1+$difference"/>
		</xsl:call-template>
		
		<xsl:if test="number($pageNumber)+1 &lt; $displayPage1">
			<a>
				<xsl:attribute name="href"><xsl:value-of select="$clientParameters" />&amp;pageNumber=<xsl:value-of select="number($pageNumber)+1" />
				</xsl:attribute>
				Next &gt;&gt;
			</a>
		</xsl:if>
		
		</div>
	</xsl:template>
</xsl:stylesheet>
